; PODPROGRAMY BUFOROWANEJ OBSUGI INTERFEJSU RS-232
; Rafa Baranowski (C) 2004
;
; Etykiety procedur:
; 	Ini_RS		- inicjalizacja interfejsu
;	Odbior_RS		- obsuga bufora odbiorczego (obsuga RXC int)
;	Pobierz_kod_RS	- pobieranie wartoci z bufora odbiorczego
;	Nadawanie_RS	- obsuga bufora nadawczego (obsuga UDRE int)
;	Wyslij_kod_RS	- zapisywanie wartoci w buforze nadawczym
;
; Wymagane zmienne:
;	RS_BUF_N 	- tablica zmiennych w pamici SRAM o rozmiarze RS_ROZM_BN
;	RS_BUF_O 	- tablica zmiennych w pamici SRAM o rozmiarze RS_ROZM_BO
;	RS_WSK_WBO	- zmienna bajtowa w pamici SRAM
;	RS_WSK_RBO	- zmienna bajtowa w pamici SRAM
;	RS_WSK_WBN	- zmienna bajtowa w pamici SRAM
;	RS_WSK_RBN	- zmienna bajtowa w pamici SRAM
;
; Wymagane stae:
;	OSC_FREQ	- czstotliwo rezonatora kwarcowego [Hz]
;	RS_BAUD	- prdko transmisji [bps]
;	RS_ROZM_BO	- rozmiar bufora odbiorczego
;	RS_ROZM_BN	- rozmiar bufora nadawczego
;

.MACRO RS_zwieksz_wsk	; makro zwikszajce warto wskanika bufora koowego
				; parametry:
				;   0. zmienna wskanikowa (SRAM)
				;   1. rejestr aktualnej wartoci wskanika
				;   2. rozmiar bufora

	inc	@1			; zwiksz aktualn warto wskanika
	cpi	@1, @2		; i porwnaj j z rozmiarem bufora
	brlo	Pomin			; jeli wskanik wskazuje na koniec bufora,
	ldi	@1, 0			;   to przesu go na pocztek
	Pomin:
	sts	@0, @1		; zapisz now warto zmiennej wskanikowej
.ENDM

.MACRO RS_ustal_adr	; makro zapisujce adres wybranej komrki tablicy
				;   w okrelonym rejestrze indeksowym
				; parametry:
				;   0. rejestr indeksowy (X, Y lub Z)
				;   1. adres tablicy
				;   2. rejestr wskanika tablicy

	ldi	@0L, low(@1)	; aduj modsz cz adresu do rejestru indeksowego
	ldi	@0H, high(@1)	; aduj starsz cz adresu do rejestru indeksowego
	add	@0L, @2		; dodaj warto wskanika do modszej czci r. ind.
	brcc	Pomin			; jeli wystpio przeniesienie, to
	inc	@0H			;   zwiksz starsz cz rejestru indeksowego
	Pomin:
.ENDM

;****************************************************************
;		PROCEDURA INICJALIZUJCA INTERFEJS RS-232
;
; Ustawienia interfejsu:
;	- 8-bitowa dugo sowa transmisyjnego,
;	- 1 bit stopu,
;	- brak bitu parzystoci,
;	- brak sterowania przepywem.
;
; Wymagane stae:
;	OSC_FREQ	- czstotliwo rezonatora kwarcowego [Hz]
;	RS_BAUD	- prdko transmisji [bps]
;
Ini_RS:
	push	R16

	ldi	R16, high(OSC_FREQ/(16*RS_BAUD)-1)
	out	UBRRH, R16
	ldi	R16, low(OSC_FREQ/(16*RS_BAUD)-1)
	out	UBRRL, R16		; ustalanie prdkoci transmisji

	ldi	R16, (1<<RXEN)|(1<<TXEN)|(1<<RXCIE)|(1<<UDRIE)
	out	UCSRB, R16		; nadajnik i odbiornik uaktywnione, wczenie
					;   generacji przerwa przy zakoczeniu odbioru
					;   i przy zakoczeniu nadawania
	ldi	R16, (1<<URSEL)|(1<<UCSZ1)|(1<<UCSZ0)
	out	UCSRC, R16		; interfejs asynchroniczny, 8-bitowa dugo sowa
					;   1 bit stopu, brak bitu parzystoci

	clr	R16
	sts	RS_WSK_WBO, R16
	sts	RS_WSK_RBO, R16
	sts	RS_WSK_WBN, R16
	sts	RS_WSK_RBN, R16	; inicjalizacja wskanikw bufora odb. i nad.

	pop	R16
	ret


;****************************************************************
;	PROCEDURA OBSUGI BUFORA ODBIORCZEGO RS-232 (obsuga RXC int)
;
; Wymagane stae:
;	RS_ROZM_BO	- rozmiar bufora odbiorczego
;
; Parametry:
;	RS_WSK_WBO	- wskanik zapisu bufora odbiorczego (SRAM)
;
; Wartoci wyjciowe:
;	RS_BUF_O 	- bufor odbiorczy (SRAM)
;
Odbior_RS:
	push	R16
	push	R17
	push	XL
	push	XH
	in	R16, SREG
	push	R16

	in	R16, UCSRA		; pobieranie znacznikw powodzenia transmisji
	in	R17, UDR		; pobieranie wartoci odebranej przez UART

	andi	R16, (1<<FE)|(1<<PE)
	brne	Odb_RS_koniec	; jeli w trakcie transmisji wystpi bd,
					;   to zignoruj warto odebran i zakocz
	lds	R16, RS_WSK_WBO
	RS_ustal_adr X, RS_BUF_O, R16
					; zaaduj X adresem komrki bufora odbiorczego,
					;   ktra ma zosta zapisana
	st	X, R17		; zapisz warto odebran w buforze odbiorczym 

	RS_zwieksz_wsk RS_WSK_WBO, R16, RS_ROZM_BO
					; zwiksz wskanik zapisu bufora odbiorczego
Odb_RS_koniec:
	pop	R16
	out	SREG, R16
	pop	XH
	pop	XL
	pop	R17
	pop	R16
	reti	


;****************************************************************
;	PROCEDURA POBIERAJCA WARTO Z BUFORA ODBIORCZEGO RS-232
;
; Wymagane stae:
;	RS_ROZM_BO	- rozmiar bufora odbiorczego
;
; Parametry:
;	RS_WSK_RBO	- wskanik odczytu bufora odbiorczego (SRAM)
;	RS_BUF_O 	- bufor odbiorczy (SRAM)
;
; Wartoci wyjciowe:
;	T		- znacznik powodzenia (SREG)
;	R16		- warto pobrana z bufora odbiorczego (o ile T=1)
;
Pobierz_kod_RS:
	push	R17
	push	XL
	push	XH

	lds	R16, RS_WSK_WBO
	lds	R17, RS_WSK_RBO
	cp	R16, R17		; porwnaj wskaniki zapisu i odczytu buf. odb. 
	brne	Pob_kod_RS_1	; jeli s one rwne (bufor pusty), to
	clt				;   zeruj znacznik powodzenia
	rjmp	Pob_kod_RS_koniec	;   i zakocz procedur pobierajc

Pob_kod_RS_1:
	RS_ustal_adr X, RS_BUF_O, R17
					; zaaduj X adresem komrki bufora odbiorczego,
					;   ktra ma zosta odczytana
	ld	R16, X		; zapisz w R16 warto odczytan z bufora odb.
	set				; i ustaw znacznik powodzenia

	RS_zwieksz_wsk RS_WSK_RBO, R17, RS_ROZM_BO
					; zwiksz wskanik odczytu bufora odbiorczego
Pob_kod_RS_koniec:
	pop	XH
	pop	XL
	pop	R17
	ret


;****************************************************************
;	PROCEDURA OBSUGI BUFORA NADAWCZEGO RS-232 (obsuga UDRE int)
;
; Wymagane stae:
;	RS_ROZM_BN	- rozmiar bufora nadawczego
;
; Parametry:
;	RS_WSK_RBN	- wskanik odczytu bufora nadawczego (SRAM)
;	RS_BUF_N 	- bufor nadawczy (SRAM)
;
Nadawanie_RS:
	push	R16
	push	R17
	push	XL
	push	XH
	in	R16, SREG
	push	R16

	lds	R16, RS_WSK_RBN
	lds	R17, RS_WSK_WBN
	cp	R16, R17		; porwnaj wskaniku zapisu i odczytu buf. nad.
	brne	Nad_RS_1		; jeli s one rwne (bufor pusty), to
	cbi	UCSRB, UDRIE	;   zablokuj przerwania UDRE
	rjmp	Nad_RS_koniec	;   i zakocz procedur nadawania

Nad_RS_1:
	RS_ustal_adr X, RS_BUF_N, R16
					; zaaduj X adresem komrki bufora nadawczego,
					;   ktra ma zosta odczytana
	ld	R17, X		; zapisz w R17 warto odczytan z bufora nad.
	out	UDR, R17		; i przelij j do moduu USART

	RS_zwieksz_wsk RS_WSK_RBN, R16, RS_ROZM_BN
					; zwiksz wskanik odczytu bufora nadawczego
Nad_RS_koniec:
	pop	R16
	out	SREG, R16
	pop	XH
	pop	XL
	pop	R17
	pop	R16
	reti


;****************************************************************
;	PROCEDURA ZAPISUJCA WARTO W BUFORZE NADAWCZYM RS-232
;
; Wymagane stae:
;	RS_ROZM_BN	- rozmiar bufora nadawczego
;
; Parametry:
;	R16		- warto do zapisania w buforze nadawczym
;	RS_WSK_WBN	- wskanik zapisu bufora nadawczego (SRAM)
;
; Wartoci wyjciowe:
;	RS_BUF_N 	- bufor nadawczy (SRAM)
;
Wyslij_kod_RS:
	push	R17
	push	XL
	push	XH

	lds	R17, RS_WSK_WBN
	RS_ustal_adr X, RS_BUF_N, R17
					; zaaduj X adresem komrki bufora nadawczego,
					;   ktra ma zosta zapisana
	st	X, R16		; zapisz warto przesyan w buforze nadawczym
	set				; i ustaw znacznik powodzenia

	RS_zwieksz_wsk RS_WSK_WBN, R17, RS_ROZM_BN
					; zwiksz wskanik zapisu bufora nadawczego

	sbi	UCSRB, UDRIE	; odblokuj przerwania UDRE

Wys_kod_RS_koniec:
	pop	XH
	pop	XL
	pop	R17
	reti

