; PODPROGRAMY OBSUGI 4-CYFROWEGO WYWIETLACZA MULTIPLEKSOWANEGO LED
; Rafa Baranowski (C) 2004
;
; Etykiety procedur:
; 	Ini_w7s		- inicjalizacja wywietlacza
; 	Multipleks_w7s	- przerwaniowa obsuga wywietlacza (OC0 int)
;	Pokaz_bajt_w7s	- wywietlanie wartoci bajtu
;
; Wymagane stae:
;	SYS_FREQ 	- czstotliwo pracy w MHz
;	K_W7S_SEG	- rejestr kierunku portu segmentw
;	O_W7S_SEG	- rejestr wyjciowy portu segmentw
;	K_W7S_KOL	- rejestr kierunku portu kolumn
;	O_W7S_KOL	- rejestr wyjciowy portu kolumn
;	W7S_KOL0	- nr linii dla kolumny 1. (cyfra najmodsza)
;	W7S_KOL1	- nr linii dla kolumny 2.
;	W7S_KOL2	- nr linii dla kolumny 3.
;	W7S_KOL3	- nr linii dla kolumny 4. (cyfra najstarsza)
;
; Wymagane zmienne:
;	W7s_kolumna	- zmienna bajtowa w pamici SRAM
;	W7s_cyfry	- zmienna 4-bajtowa w pamici SRAM


;*************************************************************************
;	PROCEDURA INICJALIZACJI WYWIETLACZA MULTIPLEKSOWANEGO LED
;
Ini_w7s:
	push	R16

	ldi	R16, (1<<WGM01)|(1<<CS02)|(1<<CS00)
	out	TCCR0, R16		; licznik 0. - tryb CTC, f=fosc/1024

	in	R16, TIMSK
	sbr	R16, 1<<OCIE0
	out	TIMSK, R16		; przerwanie OC0 uaktywnione

	ldi	R16, SYS_FREQ*5
	out	OCR0, R16		; czstotliwo odwieania pojedynczej
					;   cyfry rwna ok. 49 Hz
					; (przepenianie z czstotliwoci 195,3 Hz)


	ldi	R16, 0xFF
	out	K_W7S_SEG, R16	; port segmentw w trybie wyjciowym

	in	R16, O_W7S_KOL
	ori	R16, (1<<W7S_KOL0)|(1<<W7S_KOL1)|(1<<W7S_KOL2)|(1<<W7S_KOL3)
	out	O_W7S_KOL, R16	; wyjcia kolumn w stanie wysokim (nieaktywne)

	in	R16, K_W7S_KOL
	ori	R16, (1<<W7S_KOL0)|(1<<W7S_KOL1)|(1<<W7S_KOL2)|(1<<W7S_KOL3)
	out	K_W7S_KOL, R16	; linie kolumn w trybie wyjciowym

	pop	R16
	ret


;*************************************************************************
;	PRZERWANIOWA OBSLUGA WYSWIETLACZA MULTIPLEKSOWANEGO LED
;
; Parametry:
;	W7s_kolumna	- numer aktualnie aktywnej kolumny (SRAM)
;	W7s_cyfry	- 4-elementowa tablica cyfr do wywietlenia (SRAM)
;
Multipleks_w7s:
	push	R16
	push	R17
	push	ZL
	push	ZH
	in	R16, SREG
	push	R16

	lds	R16, W7s_kolumna		; odczytaj nr aktualnie wywietlanej kolumny
	inc	R16				; zwiksz nr aktualnie wywietlanej kolumny
	cpi	R16, 4
	brne	Mul_w7s_0			; jeli zakoczono cykl wywietlania, to
	ldi	R16, 0			;   wyzeruj nr aktualnie wywietlanej kolumny
Mul_w7s_0:
	sts	W7s_kolumna, R16		; zapamitaj nr aktualnie wywietlanej kolumny

	ldi	ZH, high(W7s_cyfry)
	ldi	ZL, low(W7s_cyfry)	; Z wskazuje na tablic wywietlanych cyfr
	ldi	R17, 0
	add	ZL, R16
	adc	ZH, R17			; do Z dodawany jest nr aktualnie wywietlanej
						;   kolumny
	ld	R16, Z			; R16 przyjmuje warto cyfry, ktra powinna
						;   si znale na aktualnie wywietlanej pozycji

	ldi	ZH, high(W7s_wzory_cyfr<<1)	; Z wskazuje na tablic 7-segmentowych
	ldi	ZL, low(W7s_wzory_cyfr<<1)	;   wzorw cyfr
	ldi	R17, 0
	add	ZL, R16			; do Z dodawana jest warto cyfry, ktrej wzr
	adc	ZH, R17			;   ma zosta odczytany
	lpm	R16, Z			; R16 przyjmuje warto 7-segmentowego wzoru
						;   cyfry, ktra ma zosta wywietlona 
	ldi	R17, 0x00
	out	K_W7S_SEG, R17		; wycz chwilowo bufory wyjciowe linii segm.
						;   (wygaszanie wywietlacza na czas zmiany)

	out	O_W7S_SEG, R16		; wywietl cyfr na wywietlaczu 7-segmentowym

	ldi	ZH, high(W7s_kolejnosc<<1)	; Z wskazuje na tablic kolejnoci
	ldi	ZL, low(W7s_kolejnosc<<1)	;   cyfr (wymusze na liniach kolumn)
	lds	R16, W7s_kolumna		; odczytaj nr aktualnie wywietlanej kolumny
	ldi	R17, 0
	add	ZL, R16			; do Z dodawany jest nr kolumny, ktra ma
	adc	ZH, R17			;   zosta uaktywniona
	lpm	R17, Z			; R17 przyjmuje warto nowego wymuszenia
						;   na liniach kolumnowych 
	in	R16, O_W7S_KOL
	ori	R16, (1<<W7S_KOL0)|(1<<W7S_KOL1)|(1<<W7S_KOL2)|(1<<W7S_KOL3)
	and	R16, R17
	out	O_W7S_KOL, R16		; zmiana aktywnej kolumny (wymuszenie niskiego
						;   stanu na nowej linii kolumnowej)

	ldi	R17, 0xFF
	out	K_W7S_SEG, R17		; wcz bufory wyjciowe na liniach segmentw
						;   (przywrcenie wywietlania)
	pop	R16
	out	SREG, R16
	pop	ZH
	pop	ZL
	pop	R17
	pop	R16
	reti

W7s_wzory_cyfr:		; tablica wzorw cyfr dla wywietlacza 7-segmentowego
				;   kolejno wzorw wyznaczona jest rosncymi wartociami
				;   odpowiadajcych im cyfr

; Segmenty:	   kgfedcba     kgfedcba
	.DB	~0b00111111, ~0b00000110	; 0, 1
	.DB	~0b01011011, ~0b01001111	; 2, 3
	.DB	~0b01100110, ~0b01101101	; 4, 5
	.DB	~0b01111101, ~0b00000111	; 6, 7
	.DB	~0b01111111, ~0b01101111	; 8, 9
	.DB	~0b01110111, ~0b01111100	; A, B
	.DB	~0b00111001, ~0b01011110	; C, D
	.DB	~0b01111001, ~0b01110001	; E, F
	.DB	~0b01000000, ~0b00001000	; -, _
				; znak "~" oznacza negacj wszystkich bitow
				;   (segment wieci, gdy linia znajduje si w stanie niskim)

W7s_kolejnosc:		; tablica wymusze dla portu kolumn
				;   kolejno wymusze wyznaczona jest rosncymi numerami
				;   odpowiadajcych im aktywnych kolumn
				;   (warto kadego wymuszenia stanowi bajt, ktrego
				;    odpowiedni bit ma warto zerow)
	.DB	~(1<<W7S_KOL0), ~(1<<W7S_KOL1), ~(1<<W7S_KOL2), ~(1<<W7S_KOL3)


;*************************************************************************
;	KONWERSJA BAJTU NA LICZB WYWIETLAN NA WYSWIETLACZU 7-SEGM.
;
; Parametry:
;	R16		- warto do wywietlenia
;
; Wartoci wyjciowe:
;	W7s_cyfry	- 4-elementowa tablica cyfr do wywietlenia (SRAM)
;
Pokaz_bajt_w7s:
	push	R17

	ldi	R17, 0
	sts	(W7s_cyfry+3), R17	; cyfra tysicy zawsze rwna 0

	ldi	R17, 0
Pok_b_w7s_0:				; ptla wykonywana tyle razy, ile razy
	subi	R16, 100			;   od zadanej wartoci mona odj
	brlo	Pok_b_w7s_1			;   liczb 100
	inc	R17				; zwikszanie licznika ptli (liczby setek)
	rjmp	Pok_b_w7s_0
Pok_b_w7s_1:
	subi	R16, -100			; dodawanie przesadnie odjtej setki
	sts	(W7s_cyfry+2), R17	; cyfra setek rwna liczbie odejmowa
						; teraz R16 jest pozbawione czci setnych
	ldi	R17, 0
Pok_b_w7s_2:				; ptla wykonywana tyle razy, ile razy
	subi	R16, 10			;   od poprzedniej wartoci mona odj
	brlo	Pok_b_w7s_3			;   liczb 10
	inc	R17				; zwikszanie licznika ptli (liczby dziesitek)
	rjmp	Pok_b_w7s_2
Pok_b_w7s_3:
	subi	R16, -10			; dodawanie przesadnie odjtej dziesitki
	sts	(W7s_cyfry+1), R17	; cyfra dziesitek rwna liczbie odejmowa
						; teraz R16 zawiera tylko jednostki
	sts	(W7s_cyfry+0), R16	; cyfra jednoci rwna pozostaoci

	pop	R17
	ret
