; PODPROGRAMY BUFOROWANEJ OBSUGI INTERFEJSU PS/2
; Rafa Baranowski (C) 2004
;
; Etykiety procedur:
; 	Ini_PS2		- inicjalizacja interfejsu
;	Odbior_PS2		- obsuga bufora odbiorczego (obsuga INT0)
;	Pobierz_kod_PS2 	- pobieranie wartoci z bufora odbiorczego
;
; Wymagane zmienne:
;	PS2_BUF_O 	- tablica zmiennych w pamici SRAM o rozmiarze PS2_ROZM_BO
;	PS2_WSK_WBO	- zmienna bajtowa w pamici SRAM
;	PS2_WSK_RBO	- zmienna bajtowa w pamici SRAM
;
; Wymagane stae:
;	PS2_ROZM_BO - rozmiar bufora odbiorczego
;	K_PS2		- rejestr kierunku portu interfejsu
;	O_PS2		- rejestr wyjciowy portu interfejsu
;	I_PS2		- rejestr wejciowy portu interfejsu
;	PS2_CLK	- nr linii dla sygnau CLK
;	PS2_DTA	- nr linii dla sygnau DTA (INT0)
;

.MACRO PS2_czekaj_na_CLK_H	; makro oczekujce na wysoki stan linii CLK
	Czekaj:
	sbis	I_PS2, PS2_CLK
	rjmp	Czekaj
.ENDM

.MACRO PS2_czekaj_na_CLK_L	; makro oczekujce na niski stan linii CLK
	Czekaj:
	sbic	I_PS2, PS2_CLK
	rjmp	Czekaj	
.ENDM

.MACRO PS2_zwieksz_wsk	; makro zwikszajce warto wskanika bufora koowego
				; parametry:
				;   0. zmienna wskanikowa (SRAM)
				;   1. rejestr aktualnej wartoci wskanika
				;   2. rozmiar bufora

	inc	@1			; zwiksz aktualn warto wskanika
	cpi	@1, @2		; i porwnaj j z rozmiarem bufora
	brlo	Pomin			; jeli wskanik wskazuje na koniec bufora,
	ldi	@1, 0			;   to przesu go na pocztek
	Pomin:
	sts	@0, @1		; zapisz now warto zmiennej wskanikowej
.ENDM

.MACRO PS2_ustal_adr	; makro zapisujce adres wybranej komrki tablicy
				;   w okrelonym rejestrze indeksowym
				; parametry:
				;   0. rejestr indeksowy (X, Y lub Z)
				;   1. adres tablicy
				;   2. rejestr wskanika tablicy

	ldi	@0L, low(@1)	; aduj modsz cz adresu do rejestru indeksowego
	ldi	@0H, high(@1)	; aduj starsz cz adresu do rejestru indeksowego
	add	@0L, @2		; dodaj warto wskanika do modszej czci r. ind.
	brcc	Pomin			; jeli wystpio przeniesienie, to
	inc	@0H			;   zwiksz starsz cz rejestru indeksowego
	Pomin:
.ENDM


;****************************************************************
;		PROCEDURA INICJALIZUJCA INTERFEJS PS/2
;
Ini_PS2:
	push	R16

	cbi	K_PS2, PS2_CLK
	cbi	K_PS2, PS2_DTA	; linie interfejsu pracuj jako wejcia

	sbi	O_PS2, PS2_CLK
	sbi	O_PS2, PS2_DTA	; linie interfejsu pracuj z podciganiem

	in	R16, MCUCR
	sbr	R16, 1<<ISC01
	cbr	R16, 1<<ISC00
	out	MCUCR, R16		; przerwanie INT0 wywoywane zboczem opadajcym

	ldi	R16, 1<<INTF0
	out	GIFR, R16		; zeruj znacznik dania przerwania INT0

	in	R16, GICR
	sbr	R16, 1<<INT0
	out	GICR, R16		; wcz przerwanie INT0

	clr	R16
	sts	PS2_WSK_WBO, R16	; zeruj zmienne wskanikw zapisu i odczytu
	sts	PS2_WSK_RBO, R16	;   bufora odbiorczego

	pop	R16
	ret


;****************************************************************
;	PROCEDURA OBSUGI BUFORA ODBIORCZEGO PS/2 (obsuga INT0)
;
; Wymagane stae:
;	PS2_ROZM_BO	- rozmiar bufora odbiorczego
;
; Parametry:
;	PS2_WSK_WBO	- wskanik zapisu bufora odbiorczego (SRAM)
;
; Wartoci wyjciowe:
;	PS2_BUF_O 	- bufor odbiorczy (SRAM)
;
Odbior_PS2:
	push	R16
	push	R17
	push	XL
	push	XH
	in	R16, SREG
	push	R16

	PS2_czekaj_na_CLK_L	; czekaj na zbocze opadajce CLK
					;   (odbir bitu startu)
	sbic	I_PS2, PS2_DTA	; jeli nie odebrano bitu startu (DTA=1), to
	rjmp	Odb_ps2_koniec	;   zakocz procedur odbioru

	clr	R16			; inicjalizacja zmiennej wartoci odbieranej
	ldi	XL, 8			; inicjalizacja licznika ptli wykonywanej 8 razy
Odb_PS2_1:
	PS2_czekaj_na_CLK_H	; czekaj kolejno na zbocze narastajce
	PS2_czekaj_na_CLK_L	; i opadajce (odbir kolejnego bitu wartoci)
	clc
	sbic	I_PS2, PS2_DTA	; zapisz aktualny stan linii DTA do znacznika
	sec				;   C z rejestru SREG
	ror	R16			; i wsu go do rejestru wartoci odbieranej (L->P)
	dec	XL
	brne	Odb_PS2_1		; zaptlenie iteracyjne
	
	PS2_czekaj_na_CLK_H	; czekaj na kolejno zbocze narastajce
	PS2_czekaj_na_CLK_L	; i opadajce (odbir bitu parzystoci typu odd)
	in	XL, I_PS2
	bst	XL, PS2_DTA		; zapisz bit parzystoci w znaczniku T ze SREG
	
	PS2_czekaj_na_CLK_H	; czekaj na kolejno zbocze narastajce
	PS2_czekaj_na_CLK_L	; i opadajce (odbir bitu stopu)
	sbis	I_PS2, PS2_DTA	; jeli nie odebrano bitu stopu (DTA=0), to
	rjmp	Odb_PS2_koniec	; zakocz procedur odbioru
					; testowanie parzystoci:
	clr	R17			;   inicjalizacja zmiennej wyniku ("licznik jedynek")
	mov	XH, R16		;   kopiowanie wartoci odebranej do XH
	ldi	XL, 8			;   inicjalizacja licznika ptli wyk. 8 razy
Odb_PS2_2:
	lsr	XH			;   przesuwanie kopii wartoci odebranej
	brcc	Odb_PS2_3		;   jeli wysunita zostaa "jedynka", to
	inc	R17			;     zwiksz licznik jedynek
Odb_PS2_3:
	dec	XL
	brne	Odb_PS2_2		;   zaptlenie iteracyjne

	com	R17
	andi	R17, 0x01		;   na najmodszej pozycji R17 znajduje si teraz
					;   obliczony bit parzystoci (typu odd)
	clr	XL
	bld	XL, 0			;   na najmodszej pozycji XL znajduje si teraz
					;   odebrany bit parzystoci (typu odd)
	eor	R17, XL
	brne	Odb_PS2_koniec	;   jeli odebrany bit parzystoci nie zgadza si
					;     z odebranym, to zakocz procedur

	lds	R17, PS2_WSK_WBO
	PS2_ustal_adr X, PS2_BUF_O, R17
					; zaaduj X adresem komrki bufora odbiorczego,
					;   ktra ma zosta zapisana
	st	X, R16		; zapisz warto odebran do bufora odbiorczego

	PS2_zwieksz_wsk PS2_WSK_WBO, R17, PS2_ROZM_BO
					; zwiksz wskanik zapisu bufora odbiorczego
Odb_PS2_koniec:
	ldi	R16, 1<<INTF0
	out	GIFR, R16		; zeruj znacznik dania przerwania INT0

	pop	R16
	out	SREG, R16
	pop	XH
	pop	XL
	pop	R17
	pop	R16
	reti


;****************************************************************
;	PROCEDURA POBIERAJCA WARTO Z BUFORA ODBIORCZEGO PS/2
;
; Wymagane stae:
;	PS2_ROZM_BO	- rozmiar bufora odbiorczego
;
; Parametry:
;	PS2_WSK_RBO	- wskanik odczytu bufora odbiorczego (SRAM)
;	PS2_BUF_O 	- bufor odbiorczy (SRAM)
;
; Wartoci wyjciowe:
;	T		- znacznik powodzenia (SREG)
;	R16		- warto pobrana z bufora odbiorczego (o ile T=1)
;
Pobierz_kod_PS2:
	push	R17
	push	XL
	push	XH

	lds	R16, PS2_WSK_WBO
	lds	R17, PS2_WSK_RBO
	cp	R16, R17		; porwnaj wskaniki zapisu i odczytu buf. odb.
	brne	Pob_kod_PS2_1	; jeli s one rwne (bufor pusty), to
	clt				;   zeruj znacznik powodzenia
	rjmp	Pob_kod_PS2_koniec;   i zakocz procedur pobierajc

Pob_kod_PS2_1:
	PS2_ustal_adr X, PS2_BUF_O, R17
					; zaaduj X adresem komrki bufora odbiorczego,
					;   ktra ma zosta odczytana
	ld	R16, X		; zapisz w R16 warto odczytan z bufora odbiorczego
	set				; i ustaw znacznik powodzenia

	PS2_zwieksz_wsk PS2_WSK_RBO, R17, PS2_ROZM_BO
					; zwiksz wskanik odczytu bufora odbiorczego
Pob_kod_PS2_koniec:
	pop	XH
	pop	XL
	pop	R17
	ret