; Programowalny pilot RC5, obsuga interfejsw optycznych i pamici EEPROM
; Rafa Baranowski (C) 2004

.INCLUDE "m16def.inc"		; SEGMENT DEKLARACJI

.EQU OSC_FREQ	= 1000000	; czstotliwo pracy w Hz
.EQU SYS_FREQ	= OSC_FREQ/1000000; czstotliwo pracy w MHz (dla opnie)
					; stae nadajnika podczerwieni
.EQU K_IR_OUT	= DDRD	;   rejestr kierunku portu
.EQU O_IR_OUT	= PORTD	;   rejestr wyjciowy portu
.EQU IR_OUT		= 7		;   nr linii dla sygnau nadawanego
					; stae odbiornika podczerwieni
.EQU K_IR_IN	= DDRD	;   rejestr kierunku portu
.EQU O_IR_IN	= PORTD	;   rejestr wyjciowy portu
.EQU I_IR_IN	= PIND	;   rejestr wejciowy portu
.EQU IR_IN		= 3		;   nr linii dla sygnau odbieranego
					; stae klawiatury matrycowej
.EQU K_KLAW		= DDRB	;   rejestr kierunku portu
.EQU O_KLAW		= PORTB	;   rejestr wyjciowy portu
.EQU I_KLAW		= PINB	;   rejestr wejciowy portu
.EQU K_KL_WCISN	= DDRD	;   rejestr kierunku linii "kl. wcinity"
.EQU O_KL_WCISN	= PORTD	;   rejestr wyjciowy linii "kl. wcinity"
.EQU I_KL_WCISN	= PIND	;   rejestr wejciowy linii "kl. wcinity"
.EQU KL_WCISN	= 2		;   nr linii dla sygnau "kl. wcinity"
					; stae wywietlacza LCD
.EQU K_LCD		= DDRA	;   rejestr kierunku portu
.EQU O_LCD		= PORTA	;   rejestr wyjciowy portu
.EQU LCD_RS		= 0		;   nr linii dla sygnau RS
.EQU LCD_EN		= 1		;   nr linii dla sygnau EN
.EQU LCD_DB4	= 2		;   nr linii dla sygnau DB4
.EQU LCD_DB5	= 3		;   nr linii dla sygnau DB5
.EQU LCD_DB6	= 4		;   nr linii dla sygnau DB6
.EQU LCD_DB7	= 5		;   nr linii dla sygnau DB7
					; symboliczne nazwy rejestrw roboczych
.DEF R_ADRES	= R20		;   rejestr adresu nadanej/odebranej wiadomoci RC5
.DEF R_ROZKAZ	= R21		;   rejestr rozkazu nadanej/odebranej wiadomoci RC5


.DSEG					; SEGMENT DANYCH
.ORG	0x0060			; pocztek pamici SRAM
Klaw_inf:	.BYTE	1		; zmienna klawiatury matrycowej
IR_adres:	.BYTE 1		; zmienna odebranego adresu wiadomoci RC5
IR_rozkaz:	.BYTE 1		; zmienna odebranego rozkazu wiadomoci RC5
IR_znacz:	.BYTE 1		; zmienna znacznikw wiadomoci RC5


.ESEG					; SEGMENT PAMICI EEPROM
.ORG	0x000				; pocztek pamici EEPROM
					; deklaracje wstpnych wiadomoci RC5, przyporzdkowanych
					;   kolejnym przyciskom klawiatury matrycowej
;		/---------------------  adres RC5
;		|  /------------------  rozkaz RC5
Kl_0:	.DB	0, 0;-----------------  dla danego klawisza
Kl_1:	.DB	0, 1
Kl_2:	.DB	0, 2
Kl_3:	.DB	0, 3
Kl_4:	.DB	0, 4
Kl_5:	.DB	0, 5
Kl_6:	.DB	0, 6
Kl_7:	.DB	0, 7
Kl_8:	.DB	0, 8
Kl_9:	.DB	0, 9
Kl_A:	.DB	0, 10
Kl_B:	.DB	0, 11
Kl_C:	.DB	0, 12
Kl_D:	.DB	0, 13
Kl_E:	.DB	0, 14


.MACRO ldi_wsk			; makro adujce wskanik 16-bitowy sta 16-bitow
					; parametry:
					;   0. modyfikowany wskanik (X, Y lub Z)
					;   1. 16-bitowa bitowa warto bezporednia
	ldi	@0H, high(@1)
	ldi	@0L, low(@1)
.ENDM


.CSEG					; SEGMENT PROGRAMU
.ORG	0				; wektor zerowania
	jmp	Reset			; skok do procedury inicjalizujcej
.ORG	INT0addr			; wektor przerwania INT0
	jmp	Odczyt_klawiatury	; skok do procedury odczytujcej stan klawiatury
.ORG	INT1addr			; wektor przerwania INT1
	jmp	Odbior_IR		; skok do procedury obsugujcej odbiornik podczerwieni

.ORG	0x30				; przejcie do pierwszej komrki za wektorami przerwa


					; deklaracje staych w pamici programu
L_powitanie:.DB	"   Pilot RC5", ZNAK_KONCA	; tekst powitalny
L_adres:	.DB	"A: ", ZNAK_KONCA			; etykieta adresu RC5
L_komenda:	.DB	" ", 0x3B, " C: ", ZNAK_KONCA	; etykieta rozkazu RC5
L_prog:	.DB	"Kl.programowany?", ZNAK_KONCA; zachta do programowania


Reset:				; podprogram inicjalizujcy
	ldi_wsk X, RAMEND
	out	SPH, XH
	out	SPL, XL		; inicjalizacja wskanika stosu

	rcall	Ini_IR		; inicjalizacja nadajnika i odbiornika podczerwieni
	rcall	Ini_klawiatury	; inicjalizacja klawiatury matrycowej
	rcall	Ini_LCD		; inicjalizacja wywietlacza LCD

	ldi	R16, (1<<LCD_STER)|(1<<LCD_SW_ON)
	rcall	Instrukcja_LCD	; wywietlacz LCD wczony, kursor nieaktywny

	ldi_wsk Z, L_powitanie<<1
	rcall	Pokaz_ciag_LCD	; wywietlenie powitania ("Pilot RC5")

	ldi	R17, 100
	ldi	R16, 10
	rcall	Czekaj_ms		; opnienie o 1 s

	rcall	Czysc_LCD		; czyszczenie wywietlacza

	sei				; odblokowanie przerwa


Petla:				; ptla gwna programu
					; odbir wiadomoci RC5
	lds	R16, IR_ZNACZ
	sbrs	R16, IR_NOWY	; jeli odebrano now wiadomo RC5, to..
	rjmp	Petla_pilot

	andi	R16, ~(1<<IR_NOWY)
	sts	IR_ZNACZ, R16	;   zma znacznik nowej wiadomoci

	lds	R_ADRES, IR_ADRES		;   zapisz nowy adres RC5 w rejestrze
	lds	R_ROZKAZ, IR_ROZKAZ	;   zapisz nowy rozkaz RC5 w rejestrze

Petla_pokaz:			; obsuga wywietlacza LCD
	rcall	Czysc_LCD		;   zma zawarto wywietlacza LCD
			
	ldi_wsk Z, L_adres<<1
	rcall	Pokaz_ciag_LCD	;   wywietl etykiet adresu RC5 ("A:")

	mov	R16, R_ADRES
	rcall	Pokaz_bajt_LCD	;   wywietl warto adresu wiadomoci RC5

	ldi_wsk Z, L_komenda<<1
	rcall	Pokaz_ciag_LCD	;   wywietl etykiet rozkazu RC5 ("C:")

	mov	R16, R_ROZKAZ
	rcall	Pokaz_bajt_LCD	;   wywietl warto rozkazu wiadomoci RC5

Petla_pilot:			; nadawanie wiadomoci RC5
	lds	R16, Klaw_inf
	sbrs	R16, 7		; jeli nacinito klawisz, to..
	rjmp	Petla

	cbr	R16, 1<<7		;   zeruj bit informujcy o przytrzymaniu klawisza

	cpi	R16, 15		;   jeli nacinito klawisz 15 ("*"), to..
	breq	Petla_pil_prog	;     przejd do procedury programowania

	mov	XL, R16		;   zapisz nr wcinitego klawisza w rejestrze XL
	lsl	XL			;   pomn go przez 2
	clr	XH
	rcall	Czytaj_EE		;   i odczytaj spod otrzymanego adresu z pamici EEPROM
	mov	R_ADRES, R16	;     adres RC5 odpowiadajcy wcinitemu klawiszowi

	inc	XL			;   inkrementuj nr klawisza (zwiksz adres w EEPROM)
	rcall	Czytaj_EE		;   i odczytaj spod otrzymanego adresu z pamici EEPROM
	mov	R_ROZKAZ, R16	;     rozkaz RC5 odpowiadajcy wcinitemu klawiszowi

	mov	R16, R_ADRES
	mov	R17, R_ROZKAZ
	rcall	Nadaj_IR		;   nadaj wiadomo RC5 odpowiadajc wcinitemu
					;     przyciskowi klawiatury matrycowej

	ldi	R16, 111
	ldi	R17, 3
	rcall	Czekaj_ms		;   wprowad opnienie o ok. 333 ms
					;   (czstotliwo powtarzania - ok. 3 Hz)

	rjmp	Petla_pokaz		;   wywietl informacje o nadanej wiadomoci RC5


Petla_pil_prog:			; programowanie pilota (po naciniciu klawisza "*")
	lds	R16, Klaw_inf
	sbrc	R16, 7
	rjmp	Petla_pil_prog	;   oczekuj na puszczenie klawisza

	ldi	R16, 40|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	;   przejd do drugiej linii wywietlacza LCD
	ldi_wsk Z, L_prog<<1
	rcall	Pokaz_ciag_LCD	;   wywietl zacht do wyboru kl. programowanego

Petla_pil_prog_1:
	lds	R16, Klaw_inf
	sbrs	R16, 7
	rjmp	Petla_pil_prog_1	;   oczekuj na nacinicie klawisza

	cbr	R16, 1<<7		;   zeruj bit informujcy o naciniciu klawisza

	rcall	Czysc_LCD		;   zma zawarto wywietlacza LCD

	cpi	R16, 15		;   jeli nacinito klawisz "*", to..
	breq	Petla_pil_koniec	;     anuluj procedur programowania

	mov	XL, R16		;   zapisz nr wcinitego klawisza w rejestrze XL
	lsl	XL			;   pomn go przez 2
	clr	XH
	mov	R16, R_ADRES
	rcall	Zapisz_EE		;   i zapisz pod otrzymanym adresem pamici EEPROM
					;     nowy adres RC5 dla wybranego klawisza
	inc	XL			;   inkrementuj nr klawisza (zwiksz adres w EEPROM)
	mov	R16, R_ROZKAZ	
	rcall Zapisz_EE		;   i zapisz pod otrzymanym adresem pamici EEPROM
					;     nowy rozkaz RC5 dla wybranego klawisza

Petla_pil_koniec:			; zakoczenie ptli
	lds	R16, Klaw_inf
	sbrc	R16, 7
	rjmp	Petla_pil_koniec	; oczekuj na puszczenie klawisza

	rjmp	Petla			; zaptl program


					; DOCZANE PODPROGRAMY
.INCLUDE "czekaj_ms.inc"	;   oczekiwanie milisekundowe
.INCLUDE "czekaj_us.inc"	;   oczekiwanie mikrosekundowe
.INCLUDE "ir_rc5.inc"		;   obsuga nadajnika i odbiornika podczerwieni
.INCLUDE "eeprom.inc"		;   obsuga wewntrznej pamici EEPROM
.INCLUDE "klaw_mat.inc"		;   obsuga klawiatury matrycowej
.INCLUDE "lcd.inc"		;   obsuga wywietlacza LCD
