; Prosty analizator widma, przetwarzanie sygnaw
; Rafa Baranowski (C) 2004

.INCLUDE "m16def.inc"	; SEGMENT DEKLARACJI

.EQU SYS_FREQ	= 16		; czstotliwo pracy w MHz (dla opnie)
					; stae wskanikw LED
.EQU K_LEDY		= DDRD	;   rejestr kierunku portu
.EQU O_LEDY		= PORTD	;   rejestr wyjciowy portu
.EQU LED_MAX	= 7		;   nr linii dla wskanika przekroczenia zakresu ADC
.EQU LED_POL	= 6		;   nr linii dla wskanika poowicznego wysterowania
					; stae wywietlacza LCD
.EQU K_LCD		= DDRB	;   rejestr kierunku portu
.EQU O_LCD		= PORTB	;   rejestr wyjciowy portu
.EQU LCD_RS		= 0		;   nr linii dla sygnau RS
.EQU LCD_EN		= 1		;   nr linii dla sygnau EN
.EQU LCD_DB4	= 2		;   nr linii dla sygnau DB4
.EQU LCD_DB5	= 3		;   nr linii dla sygnau DB5
.EQU LCD_DB6	= 4		;   nr linii dla sygnau DB6
.EQU LCD_DB7	= 5		;   nr linii dla sygnau DB7
					; stae klawiatury matrycowej
.EQU K_KLAW		= DDRC	;   rejestr kierunku portu
.EQU O_KLAW		= PORTC	;   rejestr wyjciowy portu
.EQU I_KLAW		= PINC	;   rejestr wejciowy portu
.EQU K_KL_WCISN	= DDRD	;   rejestr kierunku linii "kl. wcinity"
.EQU O_KL_WCISN	= PORTD	;   rejestr wyjciowy linii "kl. wcinity"
.EQU I_KL_WCISN	= PIND	;   rejestr wejciowy linii "kl. wcinity"
.EQU KL_WCISN	= 2		;   nr linii dla sygnau "kl. wcinity"

					; nazwy symboliczne rejestrw roboczych
.DEF N_PROBKI	= R25		;   rejestr numeru przetwarzanej prbki
.DEF N_PRAZKA	= R24		;   rejestr numeru obliczanego prka
.DEF AKUM_H		= R23		;   MSB rejestru akumulujcego
.DEF AKUM_L		= R22		;   LSB rejestru akumulujcego
.DEF AKUM_H_RE	= R23		;   MSB rejestru akumulujcego (cz rzeczywista)
.DEF AKUM_L_RE	= R22		;   LSB rejestru akumulujcego (cz rzeczywista)
.DEF AKUM_H_IM	= R21		;   MSB rejestru akumulujcego (cz urojona)
.DEF AKUM_L_IM	= R20		;   LSB rejestru akumulujcego (cz urojona)
.DEF FUNKCJA	= R19		;   rejestr przechowujcy warto funkcji sinus 
.DEF PROBKA		= R18		;   rejestr przechowujcy warto prbki
.DEF ACCB		= R17		;   rejestr uniwersalnego akumulatora (2)
.DEF ACC		= R16		;   rejestr uniwersalnego akumulatora (1)

					; nazwy symboliczne rejestrw roboczych wykorzystywanych
					; jako szybka pami danych
.DEF STAN		= R15		;   rejestr stanu aplikacji
	.EQU NR_BUF	= 0		;     bit rejestru STAN, okrelajcy numer bufora, w ktrym
					;     zapisywane maj by aktualnie pobierane prbki
					;       0 - pobieranie do bufora "Probki" (i wywietlanie)
					;       1 - pobieranie do bufora "Probki_2"
.DEF AKT_N_PROBKI	= R14		;    rejestr numeru aktualnie pobieranej prbki
.DEF AKT_N_KOMPON	= R13		;    rejestr numeru aktualnie pobieranego komponentu prbki
.DEF ILE_USR	= R12		;    rejestr wartoci okrelajcej liczb komponentw
					;      (prbek) skadajcych si na prbk w buforze
.DEF WAR_CZESC_H	= R11		;    MSB rejestru wartoci czciowej prbki
.DEF WAR_CZESC_L	= R10		;    LSB rejestru wartoci czciowej prbki



.DSEG			; SEGMENT DANYCH
.ORG	0x0060			; pocztek pamici SRAM
Klaw_inf:		.BYTE	1	; zmienna stanu klawiatury matrycowej
Probki:		.BYTE 256	; bufor dyskretnych wartoci sygnau (1)
Probki_2:		.BYTE 256	; bufor dyskretnych wartoci sygnau (2)
Fourier:		.BYTE 256	; bufor dyskretnych wartoci mocy widma
Histogram:		.BYTE 16	; bufor histogramu widmowego

.MACRO ldi_wsk			; makro adujce wskanik 16-bitowy sta 16-bitow
					; parametry:
					;   0. modyfikowany wskanik (X, Y lub Z)
					;   1. 16-bitowa bitowa warto bezporednia
	ldi	@0H, high(@1)
	ldi	@0L, low(@1)
.ENDM


.CSEG			; SEGMENT PROGRAMU
.ORG	0				; wektor zerowania
	jmp	Reset			; skocz do procedury inicjalizujcej
.ORG	INT0addr			; wektor przerwania INT0
	jmp	Odczyt_klawiatury	; skocz do procedury odczytujcej stan klawiatury
.ORG	ADCCaddr			; wektor przerwania ADCC
	jmp	Pobierz_probke	; skocz do procedury pobierajc kolejny komponent prbki

.ORG	0x30				; przejcie do pierwszej komrki za wektorami przerwa


					; deklaracje staych w pamici programu
					;   tekst powitalny
L_powitanie:.DB	"Analizator widma", ZNAK_KONCA
					;   ksztaty znakw histogramu (dla wywietlacza LCD)
Znaki_hist:	.DB	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF	; "0"
		.DB	0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0xFF	; "1"
		.DB	0x00,0x00,0x00,0x00,0x00,0xFF,0xFF,0xFF	; "2"
		.DB	0x00,0x00,0x00,0x00,0xFF,0xFF,0xFF,0xFF	; "3"
		.DB	0x00,0x00,0x00,0xFF,0xFF,0xFF,0xFF,0xFF	; "4"
		.DB	0x00,0x00,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF	; "5"
		.DB	0x00,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF	; "6"
		.DB	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF	; "7"


Reset:		; PROGRAM INICJALIZUJCY
	ldi_wsk X, RAMEND
	out	SPH, XH
	out	SPL, XL		; inicjalizuj wskanik stosu

	rcall	Ini_klawiatury	; inicjalizuj klawiatur matrycow
	rcall	Ini_LCD		; inicjalizuj wywietlacz LCD
	
	ldi_wsk Z, (Znaki_hist<<1)
	rcall	Zapisz_CGRAM	; zapisz pami CGRAM LCD znakami histogramu

	sbi	K_LEDY, LED_MAX
	sbi	K_LEDY, LED_POL	; linie doczone do wskanikw LED s wyjciami

	ldi	ACC, (1<<REFS1)|(1<<REFS0)
	out	ADMUX, ACC		; przetwornik ADC pracuje z wewntrznym rdem
					;   napicia odniesienia (2,56V)
	ldi	ACC, (1<<ADEN)|(1<<ADIE)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADATE)|(1<<ADSC)
	out	ADCSR, ACC		; przetwornik ADC pracuje w trybie wyzwalania
					;   automatycznego (Free Run), przy taktowaniu
					;   f=fosc/64=250 kHz, przerwania dla ADC s aktywne
					;   (prbkowanie fp=fosc/64/13=19,2 kHz)

	ldi	ACC, (1<<LCD_STER)|(1<<LCD_SW_ON)
	rcall	Instrukcja_LCD	; wywietlacz LCD wczony, kursor nieaktywny

	ldi_wsk Z, L_powitanie<<1
	rcall	Pokaz_ciag_LCD	; wywietl powitanie ("Analizator widma")

	ldi	ACCB, 100
	ldi	ACC, 10
	rcall	Czekaj_ms		; opnienie o 1 s

	rcall	Czysc_LCD		; czy wywietlacz

	ldi	ACC, 0
	mov	STAN, ACC
	mov	AKT_N_PROBKI, ACC
	mov	AKT_N_KOMPON, ACC
	mov	WAR_CZESC_H, ACC
	mov	WAR_CZESC_L, ACC
	sts	Klaw_inf, ACC	; inicjalizacja zmiennych w rejestrach i w SRAM

	sei				; odblokuj obsug przerwa


Petla:		; PTLA GWNA PROGRAMU
	lds	ACC, Klaw_inf
	cbr	ACC, 1<<7
	breq	Spr_klaw_0		; jeli nacinito klawisz o wartoci zerowej
	cpi	ACC, 7		;   lub wikszej ni 6, to..
	brlo	Spr_klaw_1
Spr_klaw_0:
	ldi	ACC, 1		; za, e bya to warto 1
Spr_klaw_1:
	ldi	ACCB, 1
Spr_klaw_2:				; ptla podnoszca liczb 2 do potgi wyznaczonej
	dec	ACC			;   przez warto nacinitego klawisza
	breq	Spr_klaw_3
	lsl	ACCB
	rjmp	Spr_klaw_2
Spr_klaw_3:
	mov	ILE_USR, ACCB	; zaaduj rejestr ILE_USR wynikiem potgowania
					;   (okrela on ilo komponentw tworzcych prbk)

	in	ACC, ADCSR
	sbrc	ACC, ADIE		; jeli aktualnie tworzona jest baza prbek, to..
	rjmp	Petla			;   oczekuj na zakoczenie tego procesu

	cbi	O_LEDY, LED_MAX	; zga wskanik przekroczenia zakresu ADC
	cbi	O_LEDY, LED_POL	; zga wskanik poowicznego wysterowania

	ldi	ACC, 1<<NR_BUF
	eor	STAN, ACC		; przecz aktualnie zapisywany bufor na przeciwny

	in	ACC, ADCSR
	ori	ACC, 1<<ADIE
	out	ADCSR, ACC		; i uruchom kolejny proces pobierania bazy prbek

Transformacja:	; PROGRAM REALIZUJCY TRANSFORMACJ DFT
	ldi	N_PRAZKA, 0		; inicjalizuj rejestr numeru obliczanej prbki widma
	ldi_wsk Y, Fourier	; inicjalizuj wskanik dla bufora prbek widma
Transf_1:
	ldi	N_PROBKI, 0		; inicjalizuj rejestr numeru przetwarzanej prbki sygnau
	ldi_wsk X, Probki		; inicjalizuj wskanik dla bufora prbek sygnau
	sbrs	STAN, NR_BUF	; jeli obecnie wypeniany jest bufor "Probki", to..
	inc	XH			;   zwiksz wskanik X o 256 (by wskaza na "Probki_2")
	clr	AKUM_H_RE
	clr	AKUM_L_RE
	clr	AKUM_H_IM
	clr	AKUM_L_IM		; inicjalizacja uywanych akumulatorw (czci Re i Im)
Transf_2:
	ldi	ZH, high(Sinus<<1); inicjalizuj MSB wskanika tablicy wartoci funkcji sinus,
					;   znajdujcej si w pamici programu
	mul	N_PROBKI, N_PRAZKA; przemn numer obliczanego prka przez numer
					;   aktualnie przetwarzanej prbki
	mov	ZL, R0		; LSB wskanika tablicy wartoci funkcji sinus zaaduj
					;   bajtem LSB otrzymanego wyniku mnoenia (funkcja
					;   sinus jest okresowa z okresem 256 prbek)
	lpm	FUNKCJA, Z		; pobierz poszukiwan warto funkcji sinus
	ld	PROBKA, X+		; pobierz warto przetwarzanej prbki

	fmuls	FUNKCJA, PROBKA	; pomn warto prbki przez sin ([1N.7Q] ze znakiem)
	ldi	ACC, 0		;   (ACC=0x00, jeli wynik dodatni)
	sbrc  R1, 7
	ldi	ACC, -1		;   (ACC=0xFF, jeli wynik ujemny)
	add	AKUM_L_IM, R1	; dodaj MSB wyniku do LSB akumulatora czci urojonej
	adc	AKUM_H_IM, ACC	; dodaj ewentualne przeniesienie do MSB akumulatora

	ldi	ACC, (Cosin-Sinus)<<1
	add	ZL, ACC
	ldi	ACC, 0
	adc	ZH, ACC		; przesu wskanik tablicy wartoci f-cji sin, by wskazywa
					;   na odpowiedni warto f-cji cos (+90st.=+64 prbki)
	lpm	FUNKCJA, Z		; pobierz poszukiwan warto funkcji cosinus
	
	fmuls	FUNKCJA, PROBKA	; pomn warto prbki przez cos ([1N.7Q] ze znakiem)
	ldi	ACC, 0		;   (ACC=0x00, jeli wynik dodatni)
	sbrc  R1, 7
	ldi	ACC, -1		;   (ACC=0xFF, jeli wynik ujemny)
	add	AKUM_L_RE, R1	; dodaj MSB wyniku do LSB akumulatora czci rzeczywistej
	adc	AKUM_H_RE, ACC	; dodaj ewentualne przeniesienie do MSB akumulatora
		
	inc	N_PROBKI		; zwiksz numer aktualnie przetwarzanej prbki
	brne	Transf_2		; jeli zakoczono przetwarzanie prbek, to..

	fmuls	AKUM_H_IM, AKUM_H_IM	; podnie do kwadratu cz MSB akumulatora Im
	movw	AKUM_L_IM, R0		; i przenie wynik z powrotem do niego
	fmuls	AKUM_H_RE, AKUM_H_RE	; podnie do kwadratu cz MSB akumulatora Re
	add	AKUM_L_IM, R0		; i dodaj kwadrat wartoci akumulatora Re do
	adc	AKUM_H_IM, R1		;   kwadratu wartoci akumulatora Im

	st	Y+, AKUM_L_IM	; zapamitaj LSB obliczonej wartoci mocy prka
	st	Y+, AKUM_H_IM	; zapamitaj MSB obliczonej wartoci mocy prka

	inc	N_PRAZKA		; zwiksz numer aktualnie obliczanego prka
	cpi	N_PRAZKA, 128	; jeli nie zakoczono oblicze (nr prka < 128), to
	brlo	Transf_1		;   powtrz powyszy algorytm dla kolejnego prka


Sumowanie:		; PROGRAM SUMUJCY MOCE PRKOW DLA 16-ELEMENTOWEGO HISTOGRAMU
	ldi	N_PRAZKA, 16	; inicjalizuj licznik przeliczanych kolumn
	ldi_wsk X, Fourier	; inicjalizuj wskanik dla bufora prbek widma
	ldi_wsk Y, Histogram	; inicjalizuj wskanik dla bufora kolumn histogramu

	clr	ACC
	sts	Fourier, ACC	; zeruj LSB wartoci prka skadowej staej
	sts	Fourier+1, ACC	; zeruj MSB wartoci prka skadowej staej

Sum_1:
	ldi	PROBKA, 8		; inicjalizuj rejestr numeru sumowanego prka
	clr	AKUM_H
	clr	AKUM_L		; zerowanie rejestrw akumulujcych
Sum_2:
	ld	ACC, X+
	add	AKUM_L, ACC		; dodaj kolejn, dyskretn warto mocy widma do 
	ld	ACC, X+		;   do akumulatora
	adc	AKUM_H, ACC

	dec	PROBKA
	brne	Sum_2			; zaptlenie iteracyjne (a do zsumowania 8 prkw)

	lsl	AKUM_L
	rol	AKUM_H
	lsl	AKUM_L
	rol	AKUM_H		; mnoenie zawartoci akumulatora przez 4

	sbrc	STAN, NR_BUF	; jeli aktualnie wypeniany jest bufor "Prbki"
	rjmp	Sum_3

	ld	ACC, Y
	add	AKUM_H, ACC		;   to dodaj do akumulatora warto odpowiedniego prka,
					;   pochodzc z wczeniejszego procesu przetwarzania
Sum_3:
	st	Y+, AKUM_H		; zapamitaj tak otrzyman warto kolumny histogramu

	dec	N_PRAZKA
	brne	Sum_1			; zaptlenie iteracyjne (a do przeliczenia 16 kolumn)


Wyswietlanie:	; PROGRAM PRZEDSTAWIAJCY HISTOGRAM NA WYWIETLACZU LCD
	sbrc	STAN, NR_BUF	; jeli ukad jest w trakcie wypeniania bufora "Petla_2",
	rjmp	Petla			;   to nie przeprowadzaj wywietlania

	ldi	ACC, 0|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	; umie kursor na pocztku pierwszej linii LCD

	ldi_wsk X, Histogram	; inicjalizuj wskanik dla bufora kolumn histogramu
	ldi	ACCB, 16		; inicjalizuj licznik ptli dla grnej czci histogramu
Wysw_petla_g:
	ld	ACC, X+		; odczytaj warto kolumny histogramu
	subi	ACC, 8		; pomniejsz j o 8
	brsh	Wysw_spr_g		; jeli jest mniejsza ni 8, to..
	ldi	ACC, ' '		;   na aktualnej pozycji wywietlany bdzie znak spacji
	rjmp	Wysw_znak_g		;   (supek pusty)
Wysw_spr_g:
	cpi	ACC, 8		; jeli jednak jest wiksza lub rwna 8, to..
	brlo	Wysw_znak_g
	ldi	ACC, 7		;   przyjmij, e jest rwna 7 (supek peny)
Wysw_znak_g:			
	rcall	Zapisz_LCD		; wywietl odpowiedni znak na aktualnej pozycji
	dec	ACCB
	brne	Wysw_petla_g	; zaptlenie iteracyjne dla grnej czci histogramu

	ldi	ACC, 40|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	; umie kursor na pocztku drugiej linii LCD

	ldi_wsk X, Histogram	; inicjalizuj wskanik dla bufora kolumn histogramu
	ldi	ACCB, 16		; inicjalizuj licznik ptli dla dolnej czci histogramu
Wysw_petla_d:
	ld	ACC, X+		; odczytaj warto kolumny histogramu
	cpi	ACC, 8
	brlo	Wysw_znak_d		; jeli jest ona wiksza lub rwna 8, to..
	ldi	ACC, 7		;   za, e wynosi 7 (supek peny)
Wysw_znak_d:
	rcall	Zapisz_LCD		; wywietl odpowiedni znak na aktualnej pozycji
	dec	ACCB
	brne	Wysw_petla_d	; zaptlenie iteracyjne dla dolnej czci histogramu


	rjmp	Petla			; zaptl program


Pobierz_probke:	; PROCEDURA OBSUGI PRZERWANIA ADCC, ZAPISUJCA KOLEJNE WARTOCI
			;   DYSKRETNE SYGNAU W ODPOWIEDNIM BUFORZE
	push	XH
	push	XL
	push	ACC
	in	ACC, SREG
	push	ACC

	in	XL, ADCL		; odczytaj warto LSB wyniku konwersji A/C
	in	XH, ADCH		; odczytaj warto MSB wyniku konwersji A/C

	cpi	XH, 0x03
	brlo	Pob_prob_1		; jeli zakres jest wykorzystany w poowie, to..
	sbi	O_LEDY, LED_POL	;   zapal wskanik poowicznego wysterowania
	cpi	XL, 0xFF
	brne	Pob_prob_1		; jeli zakres jest wykorzystany maksymalnie, to..
	sbi	O_LEDY, LED_MAX	;   zapal wskanik przekroczenia zakresu ADC

Pob_prob_1:
	subi	XL, low(1024/2)
	sbci	XH, high(1024/2)	; odejmij od wyniku skadow sta

	add	WAR_CZESC_L, XL	; dodaj wynik do rejestru wartoci czciowej
	adc	WAR_CZESC_H, XH	;   (sumowanie komponentw w celu urednienia sygnau
					;   i ograniczenia analizowanego pasma)
	inc	AKT_N_KOMPON	; zwiksz licznik komponentw aktualnej prbki
	cp	AKT_N_KOMPON, ILE_USR	; jeli prbka nie jest jeszcze pena, to..
	brlo	Pob_prob_koniec		;   zakocz biec procedur

	mov	ACC, ILE_USR
	lsl	ACC
Pob_prob_2:				; ptla dzielca otrzyman prbk przez czterokrotnie
	lsr	WAR_CZESC_H		;   zwikszon liczb komponentw, ktre j stworzyy
	ror	WAR_CZESC_L		;   (zawarto ILE_USR jest zawsze potg liczby 2)
	lsr	ACC
	brne	Pob_prob_2

	ldi_wsk X, Probki		; inicjalizuj wskanik dla bufora prbek sygnau
	sbrc	STAN, NR_BUF	; jeli obecnie wypeniany jest bufor "Probki_2", to..
	inc	XH			;   zwiksz wskanik X o 256 (by wskaza na "Probki_2")

	clr	ACC
	add	XL, AKT_N_PROBKI
	adc	XH, ACC		; zwiksz wskanik o numer aktualnie pobranej prbki

	st	X, WAR_CZESC_L	; zapisz LSB otrzymanej prbki w buforze

	clr	AKT_N_KOMPON	; zeruj numer aktualnie pobieranego komponentu
	clr	WAR_CZESC_H
	clr	WAR_CZESC_L		; zeruj warto czciow prbki

	inc	AKT_N_PROBKI
	brne	Pob_prob_koniec	; jeli pobrano ostatni prbk, to..

	clr	AKT_N_KOMPON	;   zeruj numer aktualnie pobieranego komponentu
	clr	WAR_CZESC_H
	clr	WAR_CZESC_L		;   zeruj warto czciow prbki

	
	in	ACC, ADCSR
	andi	ACC, ~(1<<ADIE)
	out	ADCSR, ACC		;   i wstrzymaj dalszy proces pobierania prbek

Pob_prob_koniec:
	pop	ACC
	out	SREG, ACC
	pop	ACC
	pop	XL
	pop	XH
	reti


			; DOCZANE PODPROGRAMY
.INCLUDE "czekaj_ms.inc"	;   oczekiwanie milisekundowe
.INCLUDE "czekaj_us.inc"	;   oczekiwanie mikrosekundowe
.INCLUDE "klaw_mat.inc"		;   obsuga klawiatury matrycowej
.INCLUDE "lcd.inc"		;   obsuga wywietlacza LCD


			; Tablica dyskretnych wartoci funkcji sinusoidalnych
			;   o okresie 256 prbek w formacie [1N.7Q] ze znakiem.
			;   Tablica musi rozpoczyna si od adresu o zerowym LSB.
.ORG	0x1B00
Sinus:				; pocztek funkcji sinus
.DB q7(0.00000), q7(0.02435), q7(0.04868), q7(0.07299)
.DB q7(0.09725), q7(0.12145), q7(0.14558), q7(0.16963)
.DB q7(0.19357), q7(0.21739), q7(0.24108), q7(0.26463)
.DB q7(0.28802), q7(0.31123), q7(0.33426), q7(0.35708)
.DB q7(0.37969), q7(0.40208), q7(0.42421), q7(0.44610)
.DB q7(0.46771), q7(0.48905), q7(0.51009), q7(0.53082)
.DB q7(0.55123), q7(0.57131), q7(0.59105), q7(0.61043)
.DB q7(0.62944), q7(0.64807), q7(0.66631), q7(0.68415)
.DB q7(0.70158), q7(0.71859), q7(0.73516), q7(0.75129)
.DB q7(0.76697), q7(0.78219), q7(0.79693), q7(0.81120)
.DB q7(0.82497), q7(0.83825), q7(0.85103), q7(0.86329)
.DB q7(0.87503), q7(0.88625), q7(0.89693), q7(0.90707)
.DB q7(0.91666), q7(0.92570), q7(0.93419), q7(0.94211)
.DB q7(0.94946), q7(0.95625), q7(0.96245), q7(0.96808)
.DB q7(0.97312), q7(0.97758), q7(0.98145), q7(0.98473)
.DB q7(0.98741), q7(0.98950), q7(0.99099), q7(0.99189)
Cosin:				; pocztek funkcji cosinus
.DB q7(0.99219), q7(0.99189), q7(0.99099), q7(0.98950)
.DB q7(0.98741), q7(0.98473), q7(0.98145), q7(0.97758)
.DB q7(0.97312), q7(0.96808), q7(0.96245), q7(0.95625)
.DB q7(0.94946), q7(0.94211), q7(0.93419), q7(0.92570)
.DB q7(0.91666), q7(0.90707), q7(0.89693), q7(0.88625)
.DB q7(0.87503), q7(0.86329), q7(0.85103), q7(0.83825)
.DB q7(0.82497), q7(0.81120), q7(0.79693), q7(0.78219)
.DB q7(0.76697), q7(0.75129), q7(0.73516), q7(0.71859)
.DB q7(0.70158), q7(0.68415), q7(0.66631), q7(0.64807)
.DB q7(0.62944), q7(0.61043), q7(0.59105), q7(0.57131)
.DB q7(0.55123), q7(0.53082), q7(0.51009), q7(0.48905)
.DB q7(0.46771), q7(0.44610), q7(0.42421), q7(0.40208)
.DB q7(0.37969), q7(0.35708), q7(0.33426), q7(0.31123)
.DB q7(0.28802), q7(0.26463), q7(0.24108), q7(0.21739)
.DB q7(0.19357), q7(0.16963), q7(0.14558), q7(0.12145)
.DB q7(0.09725), q7(0.07299), q7(0.04868), q7(0.02435)
.DB q7(0.00000), q7(-0.02435), q7(-0.04868), q7(-0.07299)
.DB q7(-0.09725), q7(-0.12145), q7(-0.14558), q7(-0.16963)
.DB q7(-0.19357), q7(-0.21739), q7(-0.24108), q7(-0.26463)
.DB q7(-0.28802), q7(-0.31123), q7(-0.33426), q7(-0.35708)
.DB q7(-0.37969), q7(-0.40208), q7(-0.42421), q7(-0.44610)
.DB q7(-0.46771), q7(-0.48905), q7(-0.51009), q7(-0.53082)
.DB q7(-0.55123), q7(-0.57131), q7(-0.59105), q7(-0.61043)
.DB q7(-0.62944), q7(-0.64807), q7(-0.66631), q7(-0.68415)
.DB q7(-0.70158), q7(-0.71859), q7(-0.73516), q7(-0.75129)
.DB q7(-0.76697), q7(-0.78219), q7(-0.79693), q7(-0.81120)
.DB q7(-0.82497), q7(-0.83825), q7(-0.85103), q7(-0.86329)
.DB q7(-0.87503), q7(-0.88625), q7(-0.89693), q7(-0.90707)
.DB q7(-0.91666), q7(-0.92570), q7(-0.93419), q7(-0.94211)
.DB q7(-0.94946), q7(-0.95625), q7(-0.96245), q7(-0.96808)
.DB q7(-0.97312), q7(-0.97758), q7(-0.98145), q7(-0.98473)
.DB q7(-0.98741), q7(-0.98950), q7(-0.99099), q7(-0.99189)
.DB q7(-0.99219), q7(-0.99189), q7(-0.99099), q7(-0.98950)
.DB q7(-0.98741), q7(-0.98473), q7(-0.98145), q7(-0.97758)
.DB q7(-0.97312), q7(-0.96808), q7(-0.96245), q7(-0.95625)
.DB q7(-0.94946), q7(-0.94211), q7(-0.93419), q7(-0.92570)
.DB q7(-0.91666), q7(-0.90707), q7(-0.89693), q7(-0.88625)
.DB q7(-0.87503), q7(-0.86329), q7(-0.85103), q7(-0.83825)
.DB q7(-0.82497), q7(-0.81120), q7(-0.79693), q7(-0.78219)
.DB q7(-0.76697), q7(-0.75129), q7(-0.73516), q7(-0.71859)
.DB q7(-0.70158), q7(-0.68415), q7(-0.66631), q7(-0.64807)
.DB q7(-0.62944), q7(-0.61043), q7(-0.59105), q7(-0.57131)
.DB q7(-0.55123), q7(-0.53082), q7(-0.51009), q7(-0.48905)
.DB q7(-0.46771), q7(-0.44610), q7(-0.42421), q7(-0.40208)
.DB q7(-0.37969), q7(-0.35708), q7(-0.33426), q7(-0.31123)
.DB q7(-0.28802), q7(-0.26463), q7(-0.24108), q7(-0.21739)
.DB q7(-0.19357), q7(-0.16963), q7(-0.14558), q7(-0.12145)
.DB q7(-0.09725), q7(-0.07299), q7(-0.04868), q7(-0.02435)
.DB q7(-0.00000), q7(0.02435), q7(0.04868), q7(0.07299)
.DB q7(0.09725), q7(0.12145), q7(0.14558), q7(0.16963)
.DB q7(0.19357), q7(0.21739), q7(0.24108), q7(0.26463)
.DB q7(0.28802), q7(0.31123), q7(0.33426), q7(0.35708)
.DB q7(0.37969), q7(0.40208), q7(0.42421), q7(0.44610)
.DB q7(0.46771), q7(0.48905), q7(0.51009), q7(0.53082)
.DB q7(0.55123), q7(0.57131), q7(0.59105), q7(0.61043)
.DB q7(0.62944), q7(0.64807), q7(0.66631), q7(0.68415)
.DB q7(0.70158), q7(0.71859), q7(0.73516), q7(0.75129)
.DB q7(0.76697), q7(0.78219), q7(0.79693), q7(0.81120)
.DB q7(0.82497), q7(0.83825), q7(0.85103), q7(0.86329)
.DB q7(0.87503), q7(0.88625), q7(0.89693), q7(0.90707)
.DB q7(0.91666), q7(0.92570), q7(0.93419), q7(0.94211)
.DB q7(0.94946), q7(0.95625), q7(0.96245), q7(0.96808)
.DB q7(0.97312), q7(0.97758), q7(0.98145), q7(0.98473)
.DB q7(0.98741), q7(0.98950), q7(0.99099), q7(0.99189)
