; PODPROGRAMY OBSUGI KLAWIATURY MATRYCOWEJ
; Rafa Baranowski (C) 2004
;
; Etykiety procedur:
; 	Ini_klawiatury	- inicjalizacja klawiatury
; 	Odczyt_klawiatury - przerwaniowy odczyt stanu klawiatury (INT0 int)
;
; Wymagane stae:
;	K_KLAW - rejestr kierunku portu klawiatury
;	O_KLAW - rejestr wyjciowy portu klawiatury
;	I_KLAW - rejestr wejciowy portu klawiatury
;	K_KL_WCISN - rejestr kierunku linii "kl. wcinity"
;	O_KL_WCISN - rejestr wyjciowy linii "kl. wcinity"
;	I_KL_WCISN - rejestr wejciowy linii "kl. wcinity"
;	KL_WCISN -  nr linii dla sygnau "kl. wcinity"
;
; Wymagane zmienne:
;	Klaw_inf - zmienna bajtowa w pamici SRAM
;
; Wymagane podprogramy dodatkowe:
;	Czekaj_us - z pliku czekaj_us.inc
;	Czekaj_ms - z pliku czekaj_ms.inc


;********************************************************
;	PROCEDURA INICJALIZACJI KLAWIATURY MATRYCOWEJ
;
Ini_klawiatury:
	push	R16			; umie na stosie zawarto rejestru R16

	ldi	R16, 0xF0
	out	K_KLAW, R16		; ustawienie kierunku portu klawiatury
					; (linie K - tryb wyjciowy, W - wejciowy)

	ldi	R16, 0x0F
	out	O_KLAW, R16		; wczenie podcigania na liniach wierszowych
					;   i wystawienie niskiego stanu na wszystkich
					;   liniach kolumnowych

	cbi	K_KL_WCISN, KL_WCISN	; linia "klawisz wcinity" pracuje jako
	sbi	O_KL_WCISN, KL_WCISN	;   wejcie z wczonym podciganiem

	in	R16, MCUCR
	cbr	R16, 1<<ISC01
	sbr	R16, 1<<ISC00
	out	MCUCR, R16		; przerwanie INT0 wywoywane dowolnym zboczem

	ldi	R16, 1<<INTF0
	out	GIFR, R16		; zeruj znacznik dania przerwania INT0

	in	R16, GICR
	sbr	R16, 1<<INT0
	out	GICR, R16		; wcz przerwanie INT0

	pop	R16			; cignij ze stosu zawarto R16
	ret


;********************************************************
;	PRZERWANIOWY ODCZYT KLAWIATURY MATRYCOWEJ
;
; Wartoci wyjciowe:
;	Klaw_inf - warto aktualnie wcinitego klawisza (SRAM)
;
Odczyt_klawiatury:
	push	R16		; umieszczanie na stosie zawartoci rej. R16,
	push	R17		;   rejestru R17,
	push	ZL		;   rejestru R30,
	push	ZH		;   rejestru R31,
	in	R16, SREG
	push	R16		;   oraz rejestru SREG

	ldi	R16, 20
	ldi	R17, 1
	call	Czekaj_ms	; wywoaj procedur opniajc (t=1*20 ms)

	sbic	I_KL_WCISN, KL_WCISN	; jeli klawisz puszczono, to...
	rjmp	Odcz_kl_puszczono		;   skocz do odpowiedniego miejsca

				; testowanie linii wierszowych (kolumny w stanie niskim)
	sbis	I_KLAW, 0
	ldi	R17, 0
	sbis	I_KLAW, 1
	ldi	R17, 4
	sbis	I_KLAW, 2
	ldi	R17, 8
	sbis	I_KLAW, 3
	ldi	R17, 12
				; rejestr R17 zawiera teraz czterokrotno numeru
				;   aktywnego wiersza (0, 4, 8, lub 12), liczb WW00

	ldi	R16, 0x0F
	out	K_KLAW, R16	; zmiana kierunku dziaania portu klawiatury
				;   (linie wierszowe - wyjcia, kolumnowe - wejcia)
	ldi	R16, 0xF0
	out	O_KLAW, R16	; wymuszenie zer na liniach wierszowych i podciganie
				;   linii kolumnowych

	ldi	R16, 10
	call	Czekaj_us	; wywoaj procedur opniajc (t=100 us)

				; testowanie linii kolumnowych (wiersze w stanie niskim)
	sbis	I_KLAW, 4
	ldi	R16, 0
	sbis	I_KLAW, 5
	ldi	R16, 1
	sbis	I_KLAW, 6
	ldi	R16, 2
	sbis	I_KLAW, 7
	ldi	R16, 3
				; rejestr R16 zawiera teraz numer aktywnej
				;   kolumny (0..3), liczb 00KK

	add	R16, R17	; dodanie numeru aktywnej kolumny do czterokrotnoci
				;   numeru aktywnego wiersza (otrzymujemy liczb WWKK)

	ldi	ZH, high(Kolejnosc_klawiszy<<1)	; aduj rejestr indeksowy adresem,
	ldi	ZL, low(Kolejnosc_klawiszy<<1)	;   pod ktrym umieszczono pocztek
								;   tablicy numerw klawiszy
	clr	R17
	add	ZL, R16
	adc	ZH, R17	; dodaj zawarto R16 (WWKK) do wskanika Z (R17=0)

	lpm	R16, Z	; odczytaj nr klawisza, odpowiadajcy numerom aktywnej
				;   kolumny i wiersza (liczbie WWKK)
	sbr	R16, 1<<7	; ustaw najstarszy bit wartoci klawisza, informujc o
				;   fakcie jego wcinicia
	sts	Klaw_inf, R16	; zapisz warto kl. w odpowiednim miejscu SRAM
	rjmp	Odcz_kl_koniec	; przejd do zakoczenia procedury

Odcz_kl_puszczono:	; w przypadku, gdy klawisz by puszczony...
	lds	R16, Klaw_inf	; odczytaj numer ostatnio wcinitego klawisza
	cbr	R16, 1<<7		; zeruj w nim bit informujcy o aktywnoci
	sts	Klaw_inf, R16	; z powrotem zapisz jego numer w pamici SRAM

Odcz_kl_koniec:
	rcall	Ini_klawiatury	; ponowna inicjalizacja klawiatury

	pop	R16		; ciganie ze stosu zawartoci rej. SREG,
	out	SREG, R16
	pop	ZH		;   rejestru R31,
	pop	ZL		;   rejestru R30,
	pop	R17		;   rejestru R17,
	pop	R16		;   oraz rejestru R16
	reti

Kolejnosc_klawiszy:	; tablica wartoci klawiszy wymienionych w
				;   w kolejnoci zwikszajcej si liczby okrelajcej
				;   ich pooenie w matrycy (wg liczby WWKK)
	.DB	1, 2, 3, 10, 4, 5, 6, 11, 7, 8, 9, 12, 15, 0, 14, 13
; Odpowiadajce wartociom symbole klawiszy na pycie ZL3AVR:
;		1  2  3  A   4  5  6  B   7  8  9  C   *   0   #   D
